//
//  DigitalHumanView.h
//  DigitalHumanLib
//
//  Created by sunwen05 on 2020/5/6.
//  Copyright © 2020 sunwen05. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <BaiduRtcRoom/BaiduRtcRoom.h>
#import <DigitalHumanLib/DigitalHumanDefines.h>
#import <DigitalHumanLib/DigitalHumanRtcView.h>

NS_ASSUME_NONNULL_BEGIN
@protocol DigitalHumanDelegate;

#pragma mark -- DigitalHumanLib api
@interface DigitalHumanView : UIView

@property (nonatomic, weak)id<DigitalHumanDelegate> __nullable delegate;

/**
 设置日志打印等级（1: verbose 2: warn 3: error）
 
 @param level 日志等级
 @discussion 建议在初始化 SDK 前调用。
*/
+ (void)setLogLevel:(int)level;

/**
 获取当前sdk版本
 
 @return 版本号
 @discussion 开发者使用本 API 获取 SDK 版本号
*/
+ (NSString *)version;

/**
 数字人构造函数，传入基础配置
 
 @param backgroundImage 背景图覆盖（可选，未传不进行背景图覆盖）
 @return 数字人实例
*/
- (instancetype)initWithFrame:(CGRect)frame andBackground:(UIImageView * __nullable)backgroundImage;

/**
 登入数字人房间
 
 @param connectMessage webSocket连接成功后的第一条消息
 @param clientSetting webSocket配置
 @param rtcSetting RTC的配置
*/
- (void)login:(NSString * __nullable)connectMessage andClientSetting:(ClientConfig *)clientSetting andRtcSetting:(RtcSettings *)rtcSetting andRtcParamsSetting:(RtcParams * __nullable)rtcParamsSetting;

/**
重启数字人

 @param connectMessage webSocket连接成功后的第一条消息
 */
- (void)reboot:(NSString * __nullable)connectMessage;

/**
 登出数字人，并销毁
*/
- (void)logout;

/**
 暂停数字人
 */
- (void)pause:(BOOL)pause;

/**
 设置本地视频预览回调
 */
- (void)initLocalPreview:(UIImageView *)view;

/**
 发送消息给webSocket
 
 @param message 消息内容
*/
- (void)sendMessage:(NSString *)message;

/**
 发送消息给webSocket
 
 @param binary 二进制消息内容
*/
- (void)sendMessageWithBinary:(NSData *)binary;

/**
 开始/暂停音频推流
 */
- (void)publishAudioStreaming:(BOOL)publish;
/**
 开始/暂停视频频推流
 */
- (void)publishVideoStreaming:(BOOL)publish;

/**
 数字人静音
 
 @param mute 是否静音
*/
- (void)mute:(BOOL)mute;

/**
 RTC暂停拉流(某一会话)
 
 @param subscribe 是否暂停拉流
 @param userIds 会话id
*/
- (void)subscribeStreaming:(BOOL)subscribe byUserId:(NSArray<NSNumber *> * __nullable)userIds;

/**
 数字人暂停拉音频流
 
 @param subscribe 是否暂停拉流
*/
- (void)subscribeAudio:(BOOL)subscribe;

/**
 切换听筒和扬声器
 
 @param speakLoud 是否扬声器
*/
- (void)switchLoudSpeaker:(BOOL)speakLoud;

/**
 登录RTC
*/
- (void)loginRtcRoom;

/**
 登出RTC
*/
- (void)logoutRtcRoom;

/**
 获取BaiduRtcRoom, rtc的实例
 
 @return rtc的实例
*/
- (BaiduRtcRoomApi *)getRtcRoom;

/**
 获取RtcStatus, rtc的状态
 
 @return rtc的状态
*/
- (RtcStatus *)getRtcStatus;

@end

#pragma mark -- DigitalHumanDelegate数字人回调事件接口（继承rtc的回调事件）
@protocol DigitalHumanDelegate <DigitalHumanRtcDelegate>

/**
 数字人sdk状态变化回调
 
 @param event 状态名称
 @param data 根据状态的不同，绑定的其他数据
 */
@required
- (void)onDigitalHumanEventUpdate:(DigitalHumanEvent)event obj:(id __nullable)data;

/**
 接收webSocket消息
 
 @param message 消息内容
*/
@required
- (void)onMessage:(id)message;

/**
 在CONNECT成功后，格式化RTC参数，然后根据返回值启动RTC（可不实现）
 
 @param message CONNECT消息
 @return RTC的入参
*/
@optional
- (RtcParams *)formatRtcParams:(NSString *)message;

/**
 DigitalhumanClient 网络状态监听
 
 @param latency 处理消息时间
*/
- (void)onNetworkStatus:(long)latency;

/**
 RTC 网络状态监听
 
 @param connecStates 网络状态信息，code码越大表明越不适合进行rtc拉流
 0 或 1 表示网络抖动，可能会有轻微的卡顿；
 2 或 3 表示网络较差，可能会有明显的卡顿；
 大于等于4 表示网络非常差，不建议拉流展示人像
 */
- (void)onPeerConnectStateUpdate:(NSInteger)connecStates;
@end

NS_ASSUME_NONNULL_END
