//
//  RtcView.h
//  DigitalHumanLib
//
//  Created by sunwen05 on 2020/5/6.
//  Copyright © 2020 sunwen05. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <BaiduRtcRoom/BaiduRtcRoom.h>
#import <DigitalHumanLib/DigitalHumanDefines.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(int, DigitalHumanRtcEvent) {
    RTC_SUCCESS,
    RTC_ERROR,
    RTC_ACCESS_ERROR,
    RTC_CLOSING,
    RTC_INIT
};
typedef NS_ENUM(int, DigitalHumanRtcViewDirection) {
    RTC_VIEW_DIRECTION_START,
    RTC_VIEW_DIRECTION_CENTER,
    RTC_VIEW_DIRECTION_END
};

@protocol DigitalHumanRtcDelegate;
@protocol DigitalHumanRtcInternalDelegate;

@interface DigitalHumanRtcView : UIView
/**
 声明 DigitalHumanDelegate 的委托对象，获取lib的委托实现
*/
@property (weak, nonatomic)id<DigitalHumanRtcDelegate> __nullable delegate;
/**
 声明 DigitalHumanRtcDelegate 的委托对象，获取rtc的委托实现
*/
@property (weak, nonatomic)id<DigitalHumanRtcInternalDelegate> __nullable rtcDelegate;
/**
 声明 DigitalHumanRtcDelegate 的委托对象，获取rtc的委托实现
*/
@property (assign, nonatomic)DigitalHumanRtcViewDirection rtcViewDirection;

/**
 获取RTC的状态
 
 @return RTC的状态
*/
+ (RtcStatus *)shareStatus;
+ (void)resetStatus;

/**
 RTC构造函数，传入基础配置
 
 @param rtcSettings RTC的基础配置
 @return RTC实例
*/
- (instancetype)initWithFrame:(CGRect)frame andRtcSettings:(RtcSettings *)rtcSettings andRtcParams:(RtcParams *)rtcParamsSetting;

/**
 登录RTC房间
 
 @param rtcParams rtc房间信息
 */
- (void)login:(RtcParams *)rtcParams;

/**
 登出RTC，并销毁
*/
- (void)logout;

/**
 暂停rtc推拉流
 */
- (void)pause:(BOOL)pause;

/**
 开始/暂停音频推流
 */
- (void)publishAudioStreaming:(BOOL)publish;
/**
 开始/暂停视频频推流
 */
- (void)publishVideoStreaming:(BOOL)publish;

/**
 控制RTC静音
 
 @param mute 是否静音
*/
- (void)mute:(BOOL)mute;

/**
 RTC暂停拉流(某一会话)
 
 @param subscribe 是否暂停拉流
 @param userIds 会话id
*/
- (void)subscribeStreaming:(BOOL)subscribe byUserId:(NSArray<NSNumber *> * __nullable)userIds;

/**
 RTC暂停拉音频流
 
 @param subscribe 是否暂停拉流
*/
- (void)subscribeAudio:(BOOL)subscribe;

/**
 切换听筒和扬声器
 
 @param speakLoud 是否扬声器
*/
- (void)switchLoudSpeaker:(BOOL)speakLoud;

/**
 获取RTC的api实例
 
 @return BaiduRtcRoomApi RTC的api实例
 */
- (BaiduRtcRoomApi *)getBaiduRtcRoom;

/**
 设置本地视频预览回调
 */
- (void)initLocalPreview:(UIImageView *)view;
@end

#pragma mark -- rtc委托
/**
 RTC的外部委托
 */
@protocol DigitalHumanRtcDelegate <BaiduRtcRoomDelegate, RTCRemoteVideoViewDelegate, BaiduRtcRoomApiAudioSessionDelegate, BaiduRtcApiAudioRecordDelegate>
/**
 接收摄像头数据
 
 @param message 消息内容
*/
@optional
- (void)onCameraRecord:(UIImage *)image;

/**
 RTC 网络状态监听
 
 @param connecStates 网络状态信息，code码越大表明越不适合进行rtc拉流
 0 或 1 表示网络抖动，可能会有轻微的卡顿；
 2 或 3 表示网络较差，可能会有明显的卡顿；
 大于等于4 表示网络非常差，不建议拉流展示人像
 */
@optional
- (void)onPeerConnectStateUpdate:(NSInteger)connecStates;
@end

/**
 RTC的委托
 */
@protocol DigitalHumanRtcInternalDelegate <NSObject>

/**
 rtc状态回调
 
 @param event rtc状态类型
 @param data 状态携带参数
*/
- (void)onDigitalHumanRtcEvent:(DigitalHumanRtcEvent)event obj:(id __nullable)data;

@end

NS_ASSUME_NONNULL_END
